/*
 * Copyright (c) 1999-2000 Image Power, Inc. and the University of
 *   British Columbia.
 * Copyright (c) 2001 Michael David Adams.
 * All rights reserved.
 *
 * The complete licensing terms for this software can be found in the file
 * named "LICENSE" in this software distribution.
 */

/*
 * Math Library
 *
 * $Id$
 */

/******************************************************************************\
* Includes
\******************************************************************************/

#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <stdarg.h>

#include "memalloc.h"
#include "mathx.h"

/******************************************************************************\
* Miscellaneous Functions
\******************************************************************************/

/* Calculate the integer quantity floor(log2(x)), where x is a positive
  integer. */
int floorlog2(int x)
{
	int y;

	/* The argument must be positive. */
	assert(x > 0);

	y = 0;
	while (x > 1) {
		x >>= 1;
		++y;
	}
	return y;
}

/* Calculate the integer quantity ceil(log2(x)), where x is a positive
  integer. */
int ceillog2(int x)
{
	int y;

	/* The argument must be positive. */
	assert(x > 0);

	y = -1;
	while (x > 0) {
		x >>= 1;
		++y;
	}
	return y;
}

/* Calculate the bit position of the first leading one in a nonnegative
  integer. */
/* This function is the basically the same as ceillog2(x), except that the
  allowable range for x is slightly different. */
int firstone(int x)
{
	int n;

	/* The argument must be nonnegative. */
	assert(x >= 0);

	n = -1;
	while (x > 0) {
		x >>= 1;
		++n;
	}
	return n;
}

/* Calculate the real quantity exp2(n), where x is an integer. */
double pow2i(int n)
{
	double x;
	double a;

	x = 1.0;
	if (n < 0) {
		a = 0.5;
		n = -n;
	} else {
		a = 2.0;
	}
	while (--n >= 0) {
		x *= a;
	}
	return x;
}
