/*
 * Copyright (c) 1999-2000 Image Power, Inc. and the University of
 *   British Columbia.
 * Copyright (c) 2001 Michael David Adams.
 * All rights reserved.
 *
 * The complete licensing terms for this software can be found in the file
 * named "LICENSE" in this software distribution.
 */

/*
 * Windows Bitmap File Library
 *
 * $Id$
 */

/******************************************************************************\
* Includes.
\******************************************************************************/

#include <assert.h>

#include "types.h"
#include "stream.h"
#include "image.h"
#include "bmp.h"
#include "memalloc.h"

/******************************************************************************\
* Constructors and destructors.
\******************************************************************************/

bmp_info_t *bmp_info_create()
{
	bmp_info_t *info;
	if (!(info = malloc(sizeof(bmp_info_t)))) {
		return 0;
	}
	info->palents = 0;
	return info;
}

void bmp_info_destroy(bmp_info_t *info)
{
	if (info->palents) {
		free(info->palents);
	}
	free(info);
}

/******************************************************************************\
* Miscellaneous functions.
\******************************************************************************/

int bmp_isgrayscalepal(bmp_palent_t *palents, int numpalents)
{
	bmp_palent_t *palent;
	int i;

	for (i = numpalents, palent = palents; i > 0; --i, ++palent) {
		if (palent->red != palent->grn || palent->red != palent->blu) {
			return 0;
		}
	}
	return 1;
}

int bmp_numcmpts(bmp_info_t *info)
{
	int numcmpts;

	if (info->depth == 24) {
		numcmpts = 3;
	} else if (info->depth == 8) {
		numcmpts = bmp_isgrayscalepal(info->palents, info->numcolors) ?
		  1 : 3;
	} else {
		numcmpts = 0;
		abort();
	}
	return numcmpts;
}
