/*
 * Copyright (c) 1999-2000 Image Power, Inc. and the University of
 *   British Columbia.
 * Copyright (c) 2001 Michael David Adams.
 * All rights reserved.
 *
 * The complete licensing terms for this software can be found in the file
 * named "LICENSE" in this software distribution.
 */

/*
 * Primitive Types
 *
 * $Id$
 */

#ifndef TYPES_H
#define TYPES_H

#ifdef __cplusplus
extern "C" {
#endif

#if defined(WIN32) || defined(HAVE_WINDOWS_H)
/*
 * We are dealing with Windows and probably Visual C.  (Heaven help us.)
 * We need the types INT64 and UINT64 that are defined in the header
 * file "windows.h".
 */
#include <windows.h>
#endif

#if defined(WIN32) || defined(HAVE_WINDOWS_H)
#define uchar	unsigned char
#define	ushort	unsigned short
#define	uint	unsigned int
#define	ulong	unsigned long
#define	longlong	INT64
#define	ulonglong	UINT64
#endif

#if defined(HAVE_STDLIB_H)
#include <stdlib.h>
#endif
#if defined(HAVE_STDDEF_H)
#include <stddef.h>
#endif
#if defined(HAVE_SYS_TYPES_H)
#include <sys/types.h>
#endif

#ifdef VMS
#include <stdlib.h>
#include <stddef.h>
#include <sys/types.h>
#include <ints.h>
#define	longlong        int64
#define	ulonglong	uint64
#endif

#if defined(HAVE_STDBOOL_H)

/*
 * The C language implementation does correctly provide the standard header
 * file "stdbool.h".
 */
#include <stdbool.h>

#else

/*
 * The C language implementation does not provide the standard header file
 * "stdbool.h" as required by ISO/IEC 9899:1999.  Try to compensate for this
 * braindamage below.
 */
#if !defined(bool)
#define	bool	int
#endif

#if !defined(true)
#define true	1
#endif

#if !defined(false)
#define	false	0
#endif

#endif

#if defined(HAVE_STDINT_H)

/*
 * The C language implementation does correctly provide the standard header
 * file "stdint.h".
 */
#include <stdint.h>

#else

/*
 * The C language implementation does not provide the standard header file
 * "stdint.h" as required by ISO/IEC 9899:1999.  Try to compensate for this
 * braindamage below.
 */
#include <limits.h>
/**********/
#if !defined(INT_FAST8_MIN)
typedef signed char int_fast8_t;
#define INT_FAST8_MIN	(-127)
#define INT_FAST8_MAX	128
#endif
/**********/
#if !defined(UINT_FAST8_MIN)
typedef uchar uint_fast8_t;
#define UINT_FAST8_MIN	0
#define UINT_FAST8_MAX	255
#endif
/**********/
#if !defined(INT_FAST16_MIN)
typedef short int_fast16_t;
#define INT_FAST16_MIN	SHRT_MIN
#define INT_FAST16_MAX	SHRT_MAX
#endif
/**********/
#if !defined(UINT_FAST16_MIN)
typedef ushort uint_fast16_t;
#define UINT_FAST16_MIN	USHRT_MIN
#define UINT_FAST16_MAX	USHRT_MAX
#endif
/**********/
#if !defined(INT_FAST32_MIN)
typedef int int_fast32_t;
#define INT_FAST32_MIN	INT_MIN
#define INT_FAST32_MAX	INT_MAX
#endif
/**********/
#if !defined(UINT_FAST32_MIN)
typedef uint uint_fast32_t;
#define UINT_FAST32_MIN	UINT_MIN
#define UINT_FAST32_MAX	UINT_MAX
#endif
/**********/
#if !defined(INT_FAST64_MIN)
typedef longlong int_fast64_t;
#define INT_FAST64_MIN	LLONG_MIN
#define INT_FAST64_MAX	LLONG_MAX
#endif
/**********/
#if !defined(UINT_FAST64_MIN)
typedef ulonglong uint_fast64_t;
#define UINT_FAST64_MIN	ULLONG_MIN
#define UINT_FAST64_MAX	ULLONG_MAX
#endif
/**********/

#endif

#define	CAST(t, e) \
	((t) (e))

#ifdef __cplusplus
}
#endif

#endif
