/*
 * Copyright (c) 2001 Michael David Adams.
 * All rights reserved.
 *
 * The complete licensing terms for this software can be found in the file
 * named "LICENSE" in this software distribution.
 */

/*
 * JP2 Library
 *
 * $Id$
 */

/******************************************************************************\
* Includes.
\******************************************************************************/

#include "image.h"
#include "stream.h"
#include "memalloc.h"

#include "jp2.h"
#include "util.h"

/******************************************************************************\
* Functions.
\******************************************************************************/

image_t *image_load_jp2(stream_t *in, char *optstr)
{
	jp2_box_t *box;
	int found;
	image_t *image;

	box = 0;

	/* Get the first box.  This should be a JP box. */

	if (!(box = jp2_box_get(in))) {
		goto error;
	}
	if (box->type != JP2_BOX_JP) {
		fprintf(stderr, "expecting signature box\n");
		goto error;
	}
	if (box->data.jp.magic != JP2_JP_MAGIC) {
		fprintf(stderr, "incorrect magic number\n");
		goto error;
	}
	jp2_box_destroy(box);
	box = 0;

	/* Get the second box.  This should be a FTYP box. */
	if (!(box = jp2_box_get(in))) {
		goto error;
	}
	if (box->type != JP2_BOX_FTYP) {
		fprintf(stderr, "expecting file type box\n");
		goto error;
	}
	jp2_box_destroy(box);
	box = 0;

	found = 0;
	while ((box = jp2_box_get(in))) {
		if (getdbglevel() > 0) {
			fprintf(stderr, "box type %s\n", box->info->name);
		}
		if (box->type == JP2_BOX_JP2C) {
			found = 1;
		}
		jp2_box_destroy(box);
		box = 0;
		if (found) {
			break;
		}
	}

	if (!found) {
		fprintf(stderr, "no code stream\n");
		goto error;
	}

	if (!(image = image_load_jpc(in, optstr))) {
		goto error;
	}

	return image;
	abort();

error:
	if (box) {
		jp2_box_destroy(box);
	}
	return 0;
}

#define	VALIDATELEN	(min(JP2_JP_LEN + 16, STREAM_MAXPUTBACK))

int jp2_validate(stream_t *in)
{
	stream_t *tmpstream;
	char buf[VALIDATELEN];
	int i;
	jp2_box_t *box;

	/* Read the validation data (i.e., the data used for detecting
	  the format). */
	if (stream_read(in, buf, VALIDATELEN) != VALIDATELEN) {
		return -1;
	}

	/* Put the validation data back onto the stream, so that the
	  stream position will not be changed. */
	for (i = VALIDATELEN - 1; i >= 0; --i) {
		if (stream_ungetc(in, buf[i]) == EOF) {
			return -1;
		}
	}

	/* Create a stream containing the validation data. */
	if (!(tmpstream = stream_memopen(buf, VALIDATELEN))) {
		return -1;
	}

	/* Attempt to get a box. */
	box = jp2_box_get(tmpstream);

	/* We are finished with the stream now. */
	stream_close(tmpstream);

	if (!box) {
		/* A box could not be read. */
		return -1;
	}

	/* Ensure that a valid signature box was read. */
	if (box->type != JP2_BOX_JP || box->data.jp.magic != JP2_JP_MAGIC) {
		jp2_box_destroy(box);
		return -1;
	}

	jp2_box_destroy(box);
	return 0;
}
