/*
 * Copyright (c) 1999-2000 Image Power, Inc. and the University of
 *   British Columbia.
 * Copyright (c) 2001 Michael David Adams.
 * All rights reserved.
 *
 * The complete licensing terms for this software can be found in the file
 * named "LICENSE" in this software distribution.
 */

/*
 * Quadrature Mirror-Image Filter Bank (QMFB) Library
 *
 * $Id$
 */

#ifndef QMFB_H
#define QMFB_H

#include "seq.h"

/* The maximum number of channels for a QMF bank. */
#define	QMFB1D_MAXCHANS	2

#define	QMFB1D_RITIMODE	1

#define	QMFB1D_VERT	0x10000

#define	QMFB1D_FT	1
#define	QMFB1D_NS	2

struct qmfb1d_s;
struct qmfb1dops_s;

typedef struct {

	int start;
	int end;
	int locstart;
	int locend;

} qmfb1dband_t;

/* QMF bank */
typedef struct {
	struct qmfb1dops_s *ops;
} qmfb1d_t;

typedef struct qmfb1dops_s {
	int (*getnumchans)(qmfb1d_t *qmfb);
	int (*getanalfilters)(qmfb1d_t *qmfb, int len, seq2d_t **filters);
	int (*getsynfilters)(qmfb1d_t *qmfb, int len, seq2d_t **filters);
	void (*analyze)(qmfb1d_t *qmfb, int flags, seq2d_t *x);
	void (*synthesize)(qmfb1d_t *qmfb, int flags, seq2d_t *x);
} qmfb1dops_t;

qmfb1d_t *qmfb1d_make(int qmfbid);

/* Create a copy of a QMFB. */
qmfb1d_t *qmfb1d_copy(qmfb1d_t *qmfb);

/* Destroy a QMFB. */
void qmfb1d_destroy(qmfb1d_t *qmfb);

int qmfb1d_getnumchans(qmfb1d_t *qmfb);

int qmfb1d_getanalfilters(qmfb1d_t *qmfb, int len, seq2d_t **filters);

int qmfb1d_getsynfilters(qmfb1d_t *qmfb, int len, seq2d_t **filters);

void qmfb1d_getbands(qmfb1d_t *qmfb, int flags, uint_fast32_t xstart,
  uint_fast32_t ystart, uint_fast32_t xend, uint_fast32_t yend, int maxbands,
  int *numbandsptr, qmfb1dband_t *bands);

void qmfb1d_analyze(qmfb1d_t *qmfb, int flags, seq2d_t *x);

void qmfb1d_synthesize(qmfb1d_t *qmfb, int flags, seq2d_t *x);

#endif
