/*
 * Copyright (c) 1999-2000 Image Power, Inc. and the University of
 *   British Columbia.
 * Copyright (c) 2001 Michael David Adams.
 * All rights reserved.
 *
 * The complete licensing terms for this software can be found in the file
 * named "LICENSE" in this software distribution.
 */

/*
 * $Id$
 */

/******************************************************************************\
* Includes.
\******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>

#include "bs.h"
#include "stream.h"
#include "jpcdec.h"
#include "cs.h"
#include "mqcod.h"
#include "memalloc.h"
#include "t1cod.h"

/******************************************************************************\
* Global data.
\******************************************************************************/

int zcctxnolut[4 * 256];
int spblut[256];
int scctxnolut[256];
int magctxnolut[4096];

fix_t signmsedec[1 << NMSEDEC_BITS];
fix_t refnmsedec[1 << NMSEDEC_BITS];
fix_t signmsedec0[1 << NMSEDEC_BITS];
fix_t refnmsedec0[1 << NMSEDEC_BITS];

mqctx_t mqctxs[NUMCTXS];

/******************************************************************************\
*
\******************************************************************************/

void initmqctxs();

/******************************************************************************\
* Code.
\******************************************************************************/

int PASSTYPE(int passno)
{
	int passtype;
	switch (passno % 3) {
	case 0:
		passtype = CLNPASS;
		break;
	case 1:
		passtype = SIGPASS;
		break;
	case 2:
		passtype = REFPASS;
		break;
	default:
		passtype = -1;
		assert(0);
		break;
	}
	return passtype;
}

int NOMINALGAIN(int qmfbid, int numlvls, int lvlno, int orient)
{
if (qmfbid == JPC_COX_INS) {
	return 0;
}
	assert(qmfbid == JPC_COX_RFT);
	if (lvlno == 0) {
		assert(orient == JP_LL);
		return 0;
	} else {
		switch (orient) {
		case JP_LH:
		case JP_HL:
			return 1;
			break;
		case JP_HH:
			return 2;
			break;
		}
	}
	abort();
}

/******************************************************************************\
* Coding pass related functions.
\******************************************************************************/

int SEGTYPE(int passno, int firstpassno, int bypass)
{
	int passtype;
	if (bypass) {
		passtype = PASSTYPE(passno);
		if (passtype == CLNPASS) {
			return SEG_MQ;
		}
		return ((passno < firstpassno + 10) ? SEG_MQ : SEG_RAW);
	} else {
		return SEG_MQ;
	}
}

int SEGPASSCNT(int passno, int firstpassno, int numpasses, int bypass, int termall)
{
	int ret;
	int passtype;

	if (termall) {
		ret = 1;
	} else if (bypass) {
		if (passno < firstpassno + 10) {
			ret = 10 - (passno - firstpassno);
		} else {
			passtype = PASSTYPE(passno);
			switch (passtype) {
			case SIGPASS:
				ret = 2;
				break;
			case REFPASS:
				ret = 1;
				break;
			case CLNPASS:
				ret = 1;
				break;
			default:
				ret = -1;
				assert(0);
				break;
			}
		}
	} else {
		ret = PREC * 3 - 2;
	}
	ret = min(ret, numpasses - passno);
	return ret;
}

int ISTERMINATED(int passno, int firstpassno, int numpasses, int termall,
  int lazy)
{
	int ret;
	int n;
	if (passno - firstpassno == numpasses - 1) {
		ret = 1;
	} else {
		n = SEGPASSCNT(passno, firstpassno, numpasses, lazy, termall);
		ret = (n <= 1) ? 1 : 0;
	}

	return ret;
}

/******************************************************************************\
* Lookup table code.
\******************************************************************************/

void initluts()
{
	int i;
	int orient;
	int refine;
	float u;
	float v;
	float t;

/* XXX - hack */
initmqctxs();

	for (orient = 0; orient < 4; ++orient) {
		for (i = 0; i < 256; ++i) {
			zcctxnolut[(orient << 8) | i] = getzcctxno(i, orient);
		}
	}

	for (i = 0; i < 256; ++i) {
		spblut[i] = getspb(i << 4);
	}

	for (i = 0; i < 256; ++i) {
		scctxnolut[i] = getscctxno(i << 4);
	}

	for (refine = 0; refine < 2; ++refine) {
		for (i = 0; i < 2048; ++i) {
			magctxnolut[(refine << 11) + i] = getmagctxno((refine ? REFINE : 0) | i);
		}
	}

	for (i = 0; i < (1 << NMSEDEC_BITS); ++i) {
		t = i * pow2i(-NMSEDEC_FRACBITS);
		u = t;
		v = t - 1.5;
		signmsedec[i] = dbltofix(floor((u * u - v * v) * pow2i(NMSEDEC_FRACBITS) + 0.5) / pow2i(NMSEDEC_FRACBITS));
/* XXX - this calc is not correct */
		signmsedec0[i] = dbltofix(floor((u * u) * pow2i(NMSEDEC_FRACBITS) + 0.5) / pow2i(NMSEDEC_FRACBITS));
		u = t - 1.0;
		if (i & (1 << (NMSEDEC_BITS - 1))) {
			v = t - 1.5;
		} else {
			v = t - 0.5;
		}
		refnmsedec[i] = dbltofix(floor((u * u - v * v) * pow2i(NMSEDEC_FRACBITS) + 0.5) / pow2i(NMSEDEC_FRACBITS));
/* XXX - this calc is not correct */
		refnmsedec0[i] = dbltofix(floor((u * u) * pow2i(NMSEDEC_FRACBITS) + 0.5) / pow2i(NMSEDEC_FRACBITS));
	}
}

fix_t getsignmsedec_func(fix_t x, int bitpos)
{
	fix_t y;
	assert(!(x & (~ONES(bitpos + 1))));
	y = getsignmsedec_macro(x, bitpos);
	return y;
}

int getzcctxno(int f, int orient)
{
	int h;
	int v;
	int d;
	int n;
	int t;
	int hv;

	/* Avoid compiler warning. */
	n = 0;

	h = ((f & WSIG) != 0) + ((f & ESIG) != 0);
	v = ((f & NSIG) != 0) + ((f & SSIG) != 0);
	d = ((f & NWSIG) != 0) + ((f & NESIG) != 0) + ((f & SESIG) != 0) + ((f & SWSIG) != 0);
	switch (orient) {
	case JP_HL:
		t = h;
		h = v;
		v = t;
	case JP_LL:
	case JP_LH:
		if (!h) {
			if (!v) {
				if (!d) {
					n = 0;
				} else if (d == 1) {
					n = 1;
				} else {
					n = 2;
				}
			} else if (v == 1) {
				n = 3;
			} else {
				n = 4;
			}
		} else if (h == 1) {
			if (!v) {
				if (!d) {
					n = 5;
				} else {
					n = 6;
				}
			} else {
				n = 7;
			}
		} else {
			n = 8;
		}
		break;
	case JP_HH:
		hv = h + v;
		if (!d) {
			if (!hv) {
				n = 0;
			} else if (hv == 1) {
				n = 1;
			} else {
				n = 2;
			}
		} else if (d == 1) {
			if (!hv) {
				n = 3;
			} else if (hv == 1) {
				n = 4;
			} else {
				n = 5;
			}
		} else if (d == 2) {
			if (!hv) {
				n = 6;
			} else {
				n = 7;
			}
		} else {
			n = 8;
		}
		break;
	}
	assert(n < NUMZCCTXS);
	return ZCCTXNO + n;
}

int getspb(int f)
{
	int hc;
	int vc;
	int n;

	hc = min(((f & (ESIG | ESGN)) == ESIG) + ((f & (WSIG | WSGN)) == WSIG), 1) -
	  min(((f & (ESIG | ESGN)) == (ESIG | ESGN)) + ((f & (WSIG | WSGN)) == (WSIG | WSGN)), 1);
	vc = min(((f & (NSIG | NSGN)) == NSIG) + ((f & (SSIG | SSGN)) == SSIG), 1) -
	  min(((f & (NSIG | NSGN)) == (NSIG | NSGN)) + ((f & (SSIG | SSGN)) == (SSIG | SSGN)), 1);
	if (!hc && !vc) {
		n = 0;
	} else {
		n = (!(hc > 0 || (!hc && vc > 0)));
	}
	return n;
}

int getscctxno(int f)
{
	int hc;
	int vc;
	int n;

	/* Avoid compiler warning. */
	n = 0;

	hc = min(((f & (ESIG | ESGN)) == ESIG) + ((f & (WSIG | WSGN)) == WSIG),
	  1) - min(((f & (ESIG | ESGN)) == (ESIG | ESGN)) +
	  ((f & (WSIG | WSGN)) == (WSIG | WSGN)), 1);
	vc = min(((f & (NSIG | NSGN)) == NSIG) + ((f & (SSIG | SSGN)) == SSIG),
	  1) - min(((f & (NSIG | NSGN)) == (NSIG | NSGN)) +
	  ((f & (SSIG | SSGN)) == (SSIG | SSGN)), 1);
	assert(hc >= -1 && hc <= 1 && vc >= -1 && vc <= 1);
	if (hc < 0) {
		hc = -hc;
		vc = -vc;
	}
	if (!hc) {
		if (vc == -1) {
			n = 1;
		} else if (!vc) {
			n = 0;
		} else {
			n = 1;
		}
	} else if (hc == 1) {
		if (vc == -1) {
			n = 2;
		} else if (!vc) {
			n = 3;
		} else {
			n = 4;
		}
	}
	assert(n < NUMSCCTXS);
	return SCCTXNO + n;
}

int getmagctxno(int f)
{
	int n;
	/* XXX - This is not what the WD specifies. */

/* XXX - the WD says that the mask should not be OTHSIGMSK. */
	if (!(f & REFINE)) {
		n = (f & (OTHSIGMSK)) ? 1 : 0;
	} else {
		n = 2;
	}

	assert(n < NUMMAGCTXS);
	return MAGCTXNO + n;
}

void initctxs(mqctx_t *ctxs)
{
	mqctx_t *ctx;
	int i;

	ctx = ctxs;
	for (i = 0; i < NUMCTXS; ++i) {
		ctx->mps = 0;
		switch (i) {
		case UCTXNO:
			ctx->ind = 46;
			break;
		case ZCCTXNO:
			ctx->ind = 4;
			break;
		case AGGCTXNO:
			ctx->ind = 3;
			break;
		default:
			ctx->ind = 0;
			break;
		}
		++ctx;
	}
}

void initmqctxs()
{
	initctxs(mqctxs);
}

