/*
 * Copyright (c) 1999-2000 Image Power, Inc. and the University of
 *   British Columbia.
 * Copyright (c) 2001 Michael David Adams.
 * All rights reserved.
 *
 * The complete licensing terms for this software can be found in the file
 * named "LICENSE" in this software distribution.
 */

/*
 * Tier-2 Coding Library
 *
 * $Id$
 */

#ifndef T2COD_H
#define	T2COD_H

#include "cs.h"

typedef struct {
	int numpchgs;
	int maxpchgs;
	pchg_t **pchgs;
} pchglist_t;

typedef struct {

	int numprcs;
	int *prclyrnos;
	int prcwidthexpn;
	int prcheightexpn;
	int numhprcs;

} pirlvl_t;

typedef struct {

	int numrlvls;
	pirlvl_t *pirlvls;
	int hsamp;
	int vsamp;

} picomp_t;

typedef struct {

	int numlyrs;
	int maxrlvls;
	int numcomps;
	picomp_t *picomps;

	picomp_t *picomp;
	pirlvl_t *pirlvl;

	int compno;
	int rlvlno;
	int prcno;
	int lyrno;
	int x;
	int y;
	int xstep;
	int ystep;
	int xstart;
	int ystart;
	int xend;
	int yend;
	pchg_t *pchg;
	pchglist_t *pchglist;
	pchg_t defaultpchg;
	int pchgno;
	bool prgvolfirst;
	bool valid;

	int pktno;

} pi_t;

pi_t *pi_create0();

int pi_begin(pi_t *pi);
int pi_next(pi_t *pi);

void pi_destroy(pi_t *pi);
int pi_addpchg(pi_t *pi, jpc_pocpchg_t *pchg);

int pchglist_numstreams(pchglist_t *pchglist);
pchglist_t *pchglist_create();
int pchglist_insert(pchglist_t *pchglist, int pchgno, pchg_t *pchg);
pchg_t *pchglist_remove(pchglist_t *pchglist, int pchgno);
void pchglist_destroy(pchglist_t *pchglist);
pchg_t *pchglist_get(pchglist_t *pchglist, int pchgno);
void pchg_destroy(pchg_t *pchg);
pchg_t *pchg_copy(pchg_t *pchg);
pchglist_t *pchglist_copy(pchglist_t *pchglist);
int pchglist_numpchgs(pchglist_t *pchglist);

int pi_getind(pi_t *pi);
int pi_init(pi_t *pi);

#define pi_cmptno(pi)	(assert(pi->valid), (pi)->compno)
#define pi_rlvlno(pi)	(assert(pi->valid), (pi)->rlvlno)
#define pi_lyrno(pi)	(assert(pi->valid), (pi)->lyrno)
#define pi_prcno(pi)	(assert(pi->valid), (pi)->prcno)
#define pi_prg(pi)	(assert(pi->valid), (pi)->pchg->prgord)

#endif
