/*
 * Copyright (c) 1999-2000 Image Power, Inc. and the University of
 *   British Columbia.
 * Copyright (c) 2001 Michael David Adams.
 * All rights reserved.
 *
 * The complete licensing terms for this software can be found in the file
 * named "LICENSE" in this software distribution.
 */

/*
 * $Id$
 */

/******************************************************************************\
* Includes
\******************************************************************************/

#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <stdarg.h>

#include "memalloc.h"
#include "cs.h"
#include "tsfb.h"
#include "qmfb.h"

#include "util.h"
#include "flt.h"

/******************************************************************************\
* Miscellaneous Functions
\******************************************************************************/

tsfb_t *cod_gettsfb(int qmfbid, int numlevels)
{
	tsfb_t *tsfb;

	switch (qmfbid) {
	case JPC_COX_RFT:
		qmfbid = QMFB1D_FT;
		break;
	case JPC_COX_INS:
		qmfbid = QMFB1D_NS;
		break;
	default:
		assert(0);
		qmfbid = 10;
		break;
	}

{
	qmfb1d_t *hqmfb;
	hqmfb = qmfb1d_make(qmfbid);
	assert(hqmfb);
	tsfb = tsfb_wavelet(hqmfb, hqmfb, numlevels);
	assert(tsfb);
	qmfb1d_destroy(hqmfb);
}

	return tsfb;
}

int atoaf(char *s, int *numvalues, double **values)
{
	static char delim[] = ", \t\n";
	char buf[4096];
	int n;
	double *vs;
	char *cp;

	strncpy(buf, s, sizeof(buf));
	buf[sizeof(buf) - 1] = '\0';
	n = 0;
	if ((cp = strtok(buf, delim))) {
		++n;
		while ((cp = strtok(0, delim))) {
			if (cp != '\0') {
				++n;
			}
		}
	}

	if (n) {
		if (!(vs = malloc(n * sizeof(double)))) {
			return -1;
		}

		strncpy(buf, s, sizeof(buf));
		buf[sizeof(buf) - 1] = '\0';
		n = 0;
		if ((cp = strtok(buf, delim))) {
			vs[n] = atof(cp);
			++n;
			while ((cp = strtok(0, delim))) {
				if (cp != '\0') {
					vs[n] = atof(cp);
					++n;
				}
			}
		}
	} else {
		vs = 0;
	}

	*numvalues = n;
	*values = vs;

	return 0;
}

void hexdump(FILE *out, unsigned char *data, int len)
{
	int i;
	int j;
	for (i = 0; i < len; i += 16) {
		fprintf(out, "%04x:", i);
		for (j = 0; j < 16; ++j) {
			if (i + j < len) {
				fprintf(out, " %02x", data[i + j]);
			}
		}
		fprintf(out, "\n");
	}
}

/* Perform formatted output to standard error. */
int eprintf(const char *fmt, ...)
{
	va_list ap;
	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
	return 0;
}

/******************************************************************************\
* Debugging utilities.
\******************************************************************************/

static int dbglevel = 0;

int getdbglevel()
{
	return dbglevel;
}

int setdbglevel(int newdbglevel)
{
	int olddbglevel;
	olddbglevel = dbglevel;
	dbglevel = newdbglevel;
	return olddbglevel;
}
