/*
 * Copyright (c) 1999-2000 Image Power, Inc. and the University of
 *   British Columbia.
 * Copyright (c) 2001 Michael David Adams.
 * All rights reserved.
 *
 * The complete licensing terms for this software can be found in the file
 * named "LICENSE" in this software distribution.
 */

/*
 * Portable Pixmap/Graymap Format Support
 *
 * $Id$
 */

#ifndef PNM_H
#define PNM_H

/******************************************************************************\
* Includes.
\******************************************************************************/

#include "stream.h"
#include "image.h"

/******************************************************************************\
* Constants.
\******************************************************************************/

/* Magic numbers. */
#define PNM_MAGIC_TXTPGM	0x5032 /* Text Portable GrayMap (P2) */
#define PNM_MAGIC_TXTPPM	0x5033 /* Text Portable PixMap (P3) */
#define PNM_MAGIC_BINPGM	0x5035 /* Binary Portable GrayMap (P5) */
#define PNM_MAGIC_BINPPM	0x5036 /* Binary Portable PixMap (P6) */

/* Type of image data. */
#define PNM_TYPE_PPM	0 /* PixMap */
#define PNM_TYPE_PGM	1 /* GrayMap */

/* Format of image data. */
#define PNM_FMT_TXT	0 /* Text */
#define PNM_FMT_BIN	1 /* Binary */

#define	PNM_MAXLINELEN	79

/******************************************************************************\
* Types.
\******************************************************************************/

/* File header. */

typedef struct {

	uint_fast16_t magic;
	/* The magic number. */

	uint_fast32_t width;
	/* The image width. */

	uint_fast32_t height;
	/* The image height. */

	uint_fast32_t maxval;
	/* The maximum allowable sample value. */

} pnm_hdr_t;

/******************************************************************************\
* Functions.
\******************************************************************************/

int pnm_type(uint_fast16_t magic);
/* Determine type (i.e., PGM or PPM) from magic number. */

int pnm_fmt(uint_fast16_t magic);
/* Determine format (i.e., text or binary) from magic number. */

int pnm_maxvaltodepth(uint_fast32_t maxval);
/* Determine depth (i.e., precision) from maximum value. */

#endif
